const fs = require('fs-extra');
const path = require('path');
const { createWriteStream } = require('fs');
const archiver = require('archiver');
const { glob } = require('glob');

/**
 * Create a zip archive of a directory with ignore patterns
 * @param {string} sourceDir Source directory
 * @param {string} outputPath Output zip file path
 * @param {string[]} ignorePatterns Glob patterns to ignore
 * @returns {Promise<void>}
 */
async function createArchive(sourceDir, outputPath, ignorePatterns) {
  return new Promise((resolve, reject) => {
    // Create write stream
    const output = createWriteStream(outputPath);
    const archive = archiver('zip', {
      zlib: { level: 9 } // Maximum compression
    });

    // Listen for all archive events
    output.on('close', () => resolve());
    archive.on('error', err => reject(err));

    // Pipe archive to output file
    archive.pipe(output);

    // Get all files while respecting ignores
    const files = glob.sync('**/*', {
      cwd: sourceDir,
      dot: true,
      ignore: ignorePatterns,
      nodir: true
    });

    // Add each file to the archive
    for (const file of files) {
      const filePath = path.join(sourceDir, file);
      const stats = fs.statSync(filePath);
      
      if (stats.isFile()) {
        archive.file(filePath, { name: file });
      }
    }

    // Finalize the archive
    archive.finalize();
  });
}

module.exports = { createArchive };